/**********************************************
*函数名称: muban
*功能描述:
*输入参数:
*输出参数:
*返回值:
*其他说明:
*修改日期:          版本号      修改人     修改内容
*----------------------------------------------
*2022/12/12      v1.0 赵凌峰     初版
***********************************************/

/*
 *@Note
  Example routine to emulate a simulate USB-CDC Device, USE USART1(PA9/PA10);
  Recommended to use UART2(PA2) as debug port, you can modify the debug port
  by modifying the macro definition in debug.h


  USART3 TX PB10
         RX PB11

  GPIOA9  TX USART1  USB相通
       删除// GPIOA2  TX USART2  CAN相通
  CAN PB8  PB9
  USB CDC PA11 PA12

问题：
1.搞清楚是不是中断优先级的问题，TIM2的作用以及究竟是什么冲突
2.CAN可以接收到，但需要USB在没有插上的情况下进行发送数据，看来还是冲突问题
      插上USB，先发送USB数据可以，发送CAN数据就不行
                            如果先发送CAN数据，后面就连USB也不能发数据了
                            在先发USB的前提下，在中间发CAN，CAN接收不到，但还能继续发USB
                            在先发USB的前提下，拔掉USB，在中间发CAN，CAN接收不到，插上USB，USB无法识别
                            简称抢占先机
     没插USB，可以发送CAN
    总结还是得仔细看USB里面的函数构造是什么样的，还好之前就看了
 USB已经将状态机写的很完善了，不可能在进入USB状态后还能进行CAN收发
    只能在状态机中加入CAN收发函数


 CAN中断关闭对CAN接收没有影响
 CAN中断关闭对CAN发送没有影响

   开始拆解USB与串口的传输函数


 CAN->USB   串口输出为11位数据   IDE单独发       所以数据发送顺序为     ExtID  RTR IDE DLC Data（8or3）

*/

#include "UART.h"
#include "debug.h"
#include "usb_lib.h"



/************USART  CAN***********************/
#define TxSize1   12               //数组长度
u8 RxBuffer1[TxSize1] = "\0";      // Send by can
volatile u8 Rxfinish1 = 0;         //串口接受完成标志位
volatile u8  RxCnt1 = 0;           //接收数据实时长度
/************USART  CAN***********************/



uint8_t flag_USB_receive = 0;       //USB接收标志位
uint32_t len_USB_receive = 0;       //USB接收数据长度
//extern  volatile UART_CTL Uart;



void dec2hex(u32 n,uint8_t ID_str[9]);
void str_connect(uint8_t str1[],uint8_t str2[]);
/*********************************************************************
 * @fn      CAN_Mode_Init
 *
 * @brief   Initializes CAN communication test mode.
 *          Bps =Fpclk1/((tbs1+1+tbs2+1+1)*brp)
 *
 * @param   tsjw - CAN synchronisation jump width.
 *          tbs2 - CAN time quantum in bit segment 1.
 *          tbs1 - CAN time quantum in bit segment 2.
 *          brp - Specifies the length of a time quantum.
 *          mode - Test mode.
 *          CAN_Mode_Normal.
 *          CAN_Mode_LoopBack.
 *          CAN_Mode_Silent.
 *          CAN_Mode_Silent_LoopBack.
 *
 * @return  none
 */
void CAN_Mode_Init(u8 tsjw, u8 tbs2, u8 tbs1, u16 brp, u8 mode)
{
    GPIO_InitTypeDef      GPIO_InitSturcture = {0};
    CAN_InitTypeDef       CAN_InitSturcture = {0};
    CAN_FilterInitTypeDef CAN_FilterInitSturcture = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);

    GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitSturcture.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    CAN_InitSturcture.CAN_TTCM = DISABLE;
    CAN_InitSturcture.CAN_ABOM = DISABLE;
    CAN_InitSturcture.CAN_AWUM = DISABLE;
    CAN_InitSturcture.CAN_NART = ENABLE;
    CAN_InitSturcture.CAN_RFLM = DISABLE;
    CAN_InitSturcture.CAN_TXFP = DISABLE;
    CAN_InitSturcture.CAN_Mode = mode;
    CAN_InitSturcture.CAN_SJW = tsjw;
    CAN_InitSturcture.CAN_BS1 = tbs1;
    CAN_InitSturcture.CAN_BS2 = tbs2;
    CAN_InitSturcture.CAN_Prescaler = brp;
    CAN_Init(CAN1, &CAN_InitSturcture);

    CAN_FilterInitSturcture.CAN_FilterNumber = 0;

    /* identifier/mask mode, One 32-bit filter, StdId: 0x317 */
    CAN_FilterInitSturcture.CAN_FilterMode = CAN_FilterMode_IdMask;
    CAN_FilterInitSturcture.CAN_FilterScale = CAN_FilterScale_32bit;
    CAN_FilterInitSturcture.CAN_FilterIdHigh = 0;
    CAN_FilterInitSturcture.CAN_FilterIdLow = 0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdHigh = 0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdLow = 0;


    CAN_FilterInitSturcture.CAN_FilterFIFOAssignment = CAN_Filter_FIFO0;
    CAN_FilterInitSturcture.CAN_FilterActivation = ENABLE;
    CAN_FilterInit(&CAN_FilterInitSturcture);
}

/*********************************************************************
 * @fn      CAN_Send_Msg
 *
 * @brief   CAN Transmit function.
 *
 * @param   msg - Transmit data buffer.
 *          len - Data length.
 *
 * @return  0 - Send successful.
 *          1 - Send failed.
 */
u8 CAN_Send_Msg( u8 *msg, u8 len )
{
    u8  mbox;
    u16 i = 0;

    CanTxMsg CanTxStructure;


    CanTxStructure.ExtId = 0x317;
    CanTxStructure.IDE = CAN_Id_Extended;



    CanTxStructure.RTR = CAN_RTR_Data;
    CanTxStructure.DLC = len;

    for(i = 0; i < len; i++) {
        CanTxStructure.Data[i] = msg[i];
    }

    mbox = CAN_Transmit(CAN1, &CanTxStructure);
    i = 0;

    while((CAN_TransmitStatus(CAN1, mbox) != CAN_TxStatus_Ok) && (i < 0xFFF))
    {
        i++;
    }

    if(i == 0xFFF)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/*********************************************************************
 * @fn      CAN_Receive_Msg
 *
 * @brief   CAN Receive function.
 *
 * @param   buf - Receive data buffer.
 *          len - Data length.
 *
 * @return  CanRxStructure.DLC - Receive data length.
 */
u8 CAN_Receive_Msg( u8 *buf ,u32* ID)
{
    u8 i;

    CanRxMsg CanRxStructure={0};

    if( CAN_MessagePending( CAN1, CAN_FIFO0 ) == 0)//查看是否有消息
    {

        return 0;
    }
    printf("CAN_Receive\n");
    CAN_Receive( CAN1, CAN_FIFO0, &CanRxStructure );//读数据

    for( i=0; i<3+CanRxStructure.Data[2]; i++ )//保证数据长度，CanRxStructure.Data[2]=DLC，DLC为该帧的数据长度
    {
        buf[i] = CanRxStructure.Data[i];
    }
    buf[i] = '\0';
    *ID = CanRxStructure.ExtId;
    return CanRxStructure.DLC;

}
/*********************************************************************
 * @fn      dec2hex
 *
 * @brief   10进制转16进制，并将以0x12345678这样的形式发出，发出时为8个char类型
 *
 * @param   n - should to change the data .
 *
 * @return  void
 */
void dec2hex(u32 n,uint8_t ID_str[])
{

  char c;
  int p =0;    //数组第几个元素
  int i =0;
  int digit;
  memset(ID_str, '\0', sizeof((unsigned char *)ID_str));//先清除上一次的数据

  do{
      digit = n%16;
      ID_str[p]=digit;
      p++;
      n=n/16;

  }while(n>0);

  ID_str[p]='\0';

  for(;i<p/2;i++)//交换顺序
  {
      c=ID_str[i];
      ID_str[i]=ID_str[p-1-i];
      ID_str[p-1-i] = c;

  }

}
/*********************************************************************
 * @fn      str_connect
 *
 * @brief   将str1和str2连接起来，由于str2中有DLC（也就是表示该数组长度的数字），
 *                                  就可以根据这个参数进行数组元素的个数的参考
 *
 * @param   str1 - 第一个数组，同时也是最终存放的数组
 *          str2 - 第二个数组
 *
 * @return  void
 */
void str_connect(uint8_t str1[],uint8_t str2[])
{
    u8 i = 0;
    for(;i<8+3+str2[2];i++)
    {
        str1[i+8]=str2[i];
    }
    str1[i]='\0';
}
/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *          char * strcat ( char* destination, const char* source );
    *                                   源头字符串与目的地字符串必须都以’\0‘结束；
 *          目标空间必须足够大且可修改
 * @return  none
 */
int main(void)
{   
    /*CAN 接收数组的变量定义*/

    volatile u8 rx = 0; //接收CAN标志位
    uint8_t rxbuf[12]={'\0'};  //从CAN接收的数据
    u32 IDT = 0;        //从CAN接收的ID,仅限扩展帧，目前确定全为扩展帧
    uint8_t ID_str[22];     //ID从int转为char,并最终转换为完全体

    /*CAN 接收数组的变量定义*/


    /* CAN发送数组定义   在 UART.c   __attribute__ ((aligned(4))) uint8_t  UART1_Tx_Buf[ DEF_UARTx_TX_BUF_LEN ]; */

	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
	Delay_Init();

	USART_Printf_Init(115200);

	RCC_Configuration( );

	printf("SystemClk:%d\r\n",SystemCoreClock);
	printf("USBD SimulateCDC Demo\r\n");
    

    TIM2_Init( );

    //USARTx_CFG();//配置进入中断，中断有问题，影响TIM2的问题，修改为串口3试试
    //修改为串口3还是不行，是中断的原因吗，直接不用中断了

    //UART1_CfgInit里面有引脚定义
    //UART1_Init( 1, DEF_UARTx_BAUDRATE, DEF_UARTx_STOPBIT, DEF_UARTx_PARITY );

    //CAN speed :250K
    CAN_Mode_Init( CAN_SJW_1tq, CAN_BS2_5tq, CAN_BS1_6tq,24, CAN_Mode_Normal );

	Set_USBConfig();
    USB_Init();
 	USB_Interrupts_Config();
 	
	while(1)
	{

//       UART1_DataRx_Deal( );   //串口发送  USB接收并发送
//       UART1_DataTx_Deal( );   //USB发送  串口接收并发送         找到USB接收函数

        if(flag_USB_receive != 0 )
        {
            NVIC_DisableIRQ( USB_LP_CAN1_RX0_IRQn );
            NVIC_DisableIRQ( USB_HP_CAN1_TX_IRQn );

            CAN_Send_Msg( UART1_Tx_Buf, len_USB_receive);
            memset(UART1_Tx_Buf, '\0', sizeof(UART1_Tx_Buf));//消除数组数据
            flag_USB_receive = 0;
            len_USB_receive = 0;

            NVIC_EnableIRQ( USB_LP_CAN1_RX0_IRQn );
            NVIC_EnableIRQ( USB_HP_CAN1_TX_IRQn );
        }


        rx = CAN_Receive_Msg( rxbuf , &IDT ); //CAN接收
        dec2hex(IDT,ID_str);                  //将IDT转换为数组
        str_connect(ID_str,rxbuf);            //将字符串结合在一块
//        Delay_Ms(20);
//        printf("rx = %d\n",rx);
        if(rx)
        {

            NVIC_DisableIRQ( USB_LP_CAN1_RX0_IRQn );
            NVIC_DisableIRQ( USB_HP_CAN1_TX_IRQn );

            USBD_ENDPx_DataUp( ENDP3, &ID_str,(uint16_t)(8+3+rxbuf[2]));


            NVIC_EnableIRQ( USB_LP_CAN1_RX0_IRQn );
            NVIC_EnableIRQ( USB_HP_CAN1_TX_IRQn );

            rx = 0;
        }

	}
}







