/********************************** (C) COPYRIGHT *******************************
 * File Name          : usb_endp.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/08/08
 * Description        : Endpoint routines
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/ 
#include "usb_lib.h"
#include "usb_desc.h"
#include "usb_mem.h"
#include "hw_config.h"
#include "usb_istr.h"
#include "usb_pwr.h"
#include "usb_prop.h"
#include "UART.h"

uint8_t USBD_Endp3_Busy;
uint16_t USB_Rx_Cnt=0; 
uint8_t USBD_Endp2_Busy;


extern uint8_t flag_USB_receive;
extern uint32_t len_USB_receive;

/*********************************************************************
 * @fn      EP2_IN_Callback
 *
 * @brief  Endpoint 1 IN.
 *
 * @return  none
 */
void EP1_IN_Callback (void)
{ 
	
}



/*********************************************************************
 * @fn      EP2_OUT_Callback
 *
 * @brief  Endpoint 2 OUT.
 *
 * @return  none
 */
void EP2_OUT_Callback (void)
{ 
	uint32_t len;
	USBD_Endp2_Busy = 0;//changed by zlf
    len = GetEPRxCount( EP2_OUT & 0x7F );
//    PMAToUserBufferCopy( &UART1_Tx_Buf[ ( Uart.Tx_LoadNum * DEF_USB_FS_PACK_LEN ) ], GetEPRxAddr( EP2_OUT & 0x7F ), len );
    PMAToUserBufferCopy( &UART1_Tx_Buf[ 0 ], GetEPRxAddr( EP2_OUT & 0x7F ), len );//ÿνݶһλʼ

    flag_USB_receive = 1;
    len_USB_receive = len;
//    printf("sum = %02d \n",Uart.Tx_LoadNum * DEF_USB_FS_PACK_LEN);
    Uart.Tx_PackLen[ Uart.Tx_LoadNum ] = len;
    Uart.Tx_LoadNum++;
    if( Uart.Tx_LoadNum >= DEF_UARTx_TX_BUF_NUM_MAX - 5 -15)//
    {
        Uart.Tx_LoadNum = 0x00;

    }
    Uart.Tx_RemainNum++;

	if( Uart.Tx_RemainNum >= ( DEF_UARTx_TX_BUF_NUM_MAX - 2 - 5 - 15) )
    {
        //Uart.USB_Down_StopFlag = 0x01;
	    Uart.Tx_RemainNum = 0x00;
	    SetEPRxValid( ENDP2 );    //ÿνպҪ

    }
    else
    {
        SetEPRxValid( ENDP2 );    //ÿνպҪ
    }
}
/*********************************************************************
 * @fn      EP3_IN_Callback
 *
 * @brief  Endpoint 3 IN.
 *
 * @return  none
 */
void EP3_IN_Callback (void)
{ 
	USBD_Endp3_Busy = 0;
	Uart.USB_Up_IngFlag = 0x00;
}

/*********************************************************************
 * @fn      USBD_ENDPx_DataUp
 *
 * @brief  USBD ENDPx DataUp Function
 * 
 * @param   endp - endpoint num.
 *          *pbuf - A pointer points to data.
 *          len - data length to transmit.
 * 
 * @return  data up status.
 */
uint8_t USBD_ENDPx_DataUp( uint8_t endp, uint8_t *pbuf, uint16_t len )
{
	if( endp == ENDP3 )
	{
		if (USBD_Endp3_Busy)
		{
			return USB_ERROR;
		}
		USB_SIL_Write( EP3_IN, pbuf, len );//
		USBD_Endp3_Busy = 1;
		SetEPTxStatus( ENDP3, EP_TX_VALID );
	}
	else
	{
		return USB_ERROR;
	}
	return USB_SUCCESS;
}


/**********************************************
*:USBD_ENDTx_DataUp
*:USBպ
*: endp Ӧ˿   pbuf Ӧ
*ֵ:USB_ERROR or USB_SUCCESS Զ׼
*˵:ʱûã֪Ϊɶ
*޸:          汾      ޸     ޸
*----------------------------------------------
*2022/12/28      v1.0      
***********************************************/
uint8_t USBD_ENDTx_DataUp( uint8_t endp, uint8_t *pbuf )
{
    if( endp == ENDP2 )
    {
        if (USBD_Endp2_Busy)
        {
            return USB_ERROR;
        }
        USB_SIL_Read( EP2_OUT, pbuf );//
        USBD_Endp2_Busy = 1;
        SetEPRxStatus( ENDP2, EP_RX_VALID  );
    }
    else
    {
        return USB_ERROR;
    }
    return USB_SUCCESS;
}

